#include "RcppArmadillo.h"
#include <Rcpp.h>
// [[Rcpp::depends(RcppArmadillo)]]
using namespace Rcpp;
using namespace std; 

// [[Rcpp::export]]
Rcpp::List rq_update_cpp(arma::mat newX,
					   	arma::mat X,
					   	arma::uvec IX,
					   	arma::uvec Iy,
					   	arma::uvec Ih,
					   	arma::uvec Ihc,
					   	arma::vec beta,
					   	arma::vec Rny,
					   	unsigned int K, 
					   	unsigned int n,
					   	arma::vec xB,
					   	arma::vec P,
					   	double tau,
					   	unsigned int k,
					   	arma::uvec kk,
					   	unsigned int i,
					   	unsigned int n_in_bin,
					   	double W = 1){

	// Init 
	arma::vec xBm;

	// If weights are given - then weight the past Xny matrix
	// Notice, tho the newest observaiton is not weight - maybe need a fix?
	if(W != 1){
		X = W * X;
	}

	// X = join_cols(X, newX);

	// Rolling window of leaving the design matrix
	// i.e. the last observation leaves the matrix 
	// See Møller 2008 if something else is needed
	unsigned int Leav = 0;
	arma::uvec sortIdx = sort_index(Ih);

	//Rcout << "min" << sortIdx << "\n";
	//Rcout << "minV"  << sortIdx(0) << "\n";

	if(Ih(sortIdx(0)) == Leav){
		//Rcout << "Here but still hmm" << "\n";
		//Rcout << "Ih" << Ih << "\n";
		// Compute the inverse of X(h)
		arma::mat invXh = inv(X(Ih, IX));
		//Rcout << invXh.col(sortIdx(0)) << "\n";

		// Computing the c
		arma::vec cB = arma::conv_to<arma::vec>::from(P < 0) + P * tau;
		// Computing inverse (B^T) * c_B - P * X(bar(h)) * X(b)^-1 * inverse(X(bar(h)))
		double g = arma::as_scalar(-(P % (X(Ihc,IX) * invXh.col(sortIdx(0)))).t() * cB);
		int signG = (g > 0) - (g < 0);
		//  h is the direction or the d in the paper
		arma::vec h = join_cols(invXh.col(sortIdx(0)), -(P % (X(Ihc,IX) * invXh.col(sortIdx(0))))) * signG;
		// Compute the magitude of the step, alpha
		arma::vec sigma = arma::zeros(n - K);
		arma::vec hm = h.elem(arma::linspace<arma::uvec>(K,h.size() - 1,h.size() - K));

 		// The absolute of the residuals which are not equal to zero
		arma::vec xBm = xB.elem(arma::linspace<arma::uvec>(K,h.size()-1,h.size()-K));
		xBm.elem(find(xBm < 0)).fill(0);
		//Rcout << "Here but still hmm" << "\n";

		//Rcout << "hm" << hm << "\n";
		//Rcout << "Sigma" << sigma << "\n";

		//Rcout << "hm" << hm.size() << "\n";
		//Rcout << "Sigma" << sigma.size() << "\n";

		sigma.elem(find(hm > 1.e-10)) = xBm.elem(find(hm>1.e-10)) / hm.elem(find(hm>1.e-10));
		sigma.elem(find(hm<= 1.e-10)).fill(1.e30);

		//Rcout << "min idx " << index_min(sigma) << "\n";

		unsigned int q = index_min(sigma); //sort_index(sigma).elem(0);
	
		double alpha =  arma::as_scalar(sigma.row(q));

		arma::vec z = xB - alpha * h;
		arma::uvec Ihm = Ih.row(sortIdx(0));
		Ih.row(sortIdx(0)) = Ihc.row(q);
		Ihc(q) = Ihm(0);

		xB = z;
		xB(q+K) = alpha;

		P(q) = signG + (g == 0);
		Ih = sort(Ih);
		arma::uvec IndexIhc = sort_index(Ihc);
		Ihc = sort(Ihc);
		P = P(IndexIhc);

		xBm = xB(arma::linspace<arma::uvec>(K,xB.size()-1,xB.size()-K));
		xBm = xBm(IndexIhc);
		xB(arma::linspace<arma::uvec>(K,xB.size()-1,xB.size()-K)) = xBm;
		beta = xB(arma::linspace<arma::uvec>(0,K-1,K));
	}

	//Rcout << "IX " << IX <<  "\n";
	//Rcout << "X " << X(kk, IX) <<  "\n";
	//Rcout << "Pred " << arma::as_scalar(X(kk, IX) * beta) <<  "\n";

	double Ypred = arma::as_scalar(newX(kk, IX) * beta);
	double rny = arma::as_scalar(newX(kk, Iy)) - Ypred;

	//Rcout << "rny" << rny << "\n";
	X = join_cols(X, newX);
	
	int szR = Rny.size();
	Rny.resize(szR+1);
	Rny(szR) = rny;


	int szP = P.size();
	P.resize(szP+1);
	int szB = xB.size();
	xB.resize(szB+1);


	if(rny < 0){
		P(szP) = -1;
		xB(szB) = -rny;
	} else {
		P(szP) = 1;
		xB(szB) = rny;
	}

	int sz = Ihc.size();
	Ihc.resize(sz+1);
	Ihc(sz) = n;


	//Rcout << "Ihc" << Ihc << "\n";

	if(X.n_rows == n_in_bin){
		i += 1;
		arma::uvec stay = arma::linspace<arma::uvec>(1,P.size()-1,P.size()-1);

		P = P(stay);
		Ihc = Ihc(stay);
		//arma::vec Pnew = P(stay);
		//arma::uvec Ihcnew = Ihc(stay);

		X = X.rows(sort(join_cols(Ihc,Ih)));


		xBm = xB.elem(arma::linspace<arma::uvec>(K,xB.size()-1,xB.size()-K));
		xBm = xBm(stay);
		xB = xB.elem(arma::linspace<arma::uvec>(0,xB.size()-2,xB.size()-1));
		xB.elem(arma::linspace<arma::uvec>(K,xB.size()-1,xB.size()-K)) = xBm;
		
		//Ihc = Ihc(stay);

		Ihc = Ihc - 1;
		Ih = Ih - 1;


	} else{
		n += 1;
	}



	return List::create(Named("Ih") = Ih,
		Named("Ihc") = Ihc,
		Named("xB") = xB,
		Named("X") = X,
		Named("R") = Rny,
		Named("P") = P,
		Named("n") = n,
		Named("i") = i,
		Named("Ypred") = Ypred);

}